<?php


/**
 * Page callback for admin/dqx_adminmenu-data.js
 */
function dqx_adminmenu_js() {
  if (!user_access('use dqx_adminmenu') || $GLOBALS['user']->uid != $_GET['uid']) {
    $js = <<<EOT
      alert('dqx_adminmenu access denied.');
EOT;
  }
  else {
    $cid = 'dqx_adminmenu:'. $GLOBALS['user']->uid .':'. $GLOBALS['language']->language;
    $timestamp = variable_get('dqx_adminmenu_timestamp', 0);
    $cache = cache_get($cid)->data;
    if (!is_array($cache) || $cache['timestamp'] != $timestamp) {
      $js = _dqx_adminmenu_js();
      $cache = array('timestamp' => $timestamp, 'js' => $js);
      cache_set($cid, $cache, 'cache', CACHE_TEMPORARY);
    }
    else {
      $js = $cache['js'];
    }
  }
  
  header("Content-type: text/javascript");
  header("Cache-Control: ''"); // HTTP/1.1
  $expires = date('D, d M Y H:i:s GMT', time() + 60*60*24*2);  // two days ahead
  header("Expires: $expires");
  
  _dqx_adminmenu_js_print($js);
  
  exit();
}


function _dqx_adminmenu_js_print($js) {
  print $js;
}


function _dqx_adminmenu_js() {
  $html = _dqx_adminmenu_router_tree_html();
  
  $html = !$html ? '' : <<<EOT
  <div id="dqx_adminmenu">
    <div id="dqx_adminmenu-wrapper">$html</div>
  </div>
EOT;
  
  $json = json_encode(trim($html));
  
  return <<<EOT
    $(function(){
      var divRoot = $($json);
      $('body').prepend(divRoot);
      Drupal.dqx_adminmenu.init(divRoot);
    });
EOT;
}


function _dqx_adminmenu_router_tree_items() {
  $q = db_query("
    SELECT *
    FROM {menu_router}
    WHERE
      title != ''  AND
      (path LIKE '%s'  OR  path LIKE '%s'  OR  path = 'node/add'  OR  path = 'admin')
    ORDER BY weight ASC, title ASC
  ", 'admin/%', 'node/add/%');
  
  $items = array();
  $paths = array();
  while ($item = db_fetch_array($q)) {
    if (!strpos($item['path'], '%')) {
      if ($item['type'] & MENU_VISIBLE_IN_TREE) {
        // good
      }
      else {
        $fragments = explode('/', $item['path']);
        if (($item['type'] & MENU_VISIBLE_IN_BREADCRUMB) && (count($fragments) > 2)) {
          // good
        }
        else {
          // skip this one
          // continue;
        }
      }
      $item = _dqx_adminmenu_build_menu_item($item);
      $item['options'] = serialize($item['options']);
      $items[$item['path']] = $item;
      $paths[$item['path']] = $item['path'];
    }
  }
  
  return $items;
}


function _dqx_adminmenu_router_tree_html() {

  $items = _dqx_adminmenu_router_tree_items();
  
  unset($items['admin/by-module']);
  unset($items['admin/by-task']);
  unset($items['admin/compact']);
  
  _dqx_adminmenu_items_check_access($items);
  
  $icon_items = _dqx_adminmenu_icon_items();
  $user_items = _dqx_adminmenu_user_items();
  $menu_editor_items = _dqx_adminmenu_menu_editor_items();
  $items = array_merge($icon_items, $user_items, $items, $menu_editor_items);
  
  $items['node/add']['weight'] = 0;
  
  $paths = array_keys($items);
  $paths = array_combine($paths, $paths);
  
  $parent_map = array(
    'node' => 'admin/content',
    'admin/content/node-type' => 'admin/content/types',
    'admin/build/menu-customize' => 'admin/build/menu/list',
  );
  
  $submenus = array();
  foreach ($paths as $path => $type) {
    _dqx_adminmenu_path_multisplit($submenus, $path, $parent_map + $paths);
  }
  
  if (FALSE) {
    foreach ($items as $key => $item) {
      echo $key;
      krumo($item);
      die;
    }
  }
  
  if (module_exists('admin_views')) {
    $submenus['admin']['admin/node-types'] = 'admin/node-types';
    $submenus['admin/node-types'] = new _dqx_adminmenu_ContentTablePlugin();
    $items['admin/node-types'] = $items['admin/content/node'];
    $items['admin/node-types']['title'] = 'Node types';
    $items['admin/node-types']['weight'] = -9;
    $items['admin/build']['weight'] = -8;
  }
  
  $treeObject = new _dqx_adminmenu_MenuTree($submenus, $items);
  
  $ul_tree = $treeObject->renderMenu('admin', 'dqx_adminmenu-admin');
  $ul_user = $treeObject->renderMenu('dqx_adminmenu-user', 'dqx_adminmenu-user');
  
  if ($ul_tree) {
    return <<<EOT
    <div id="dqx_adminmenu">
      <div id="dqx_adminmenu-wrapper">
        $ul_user
        $ul_tree
      </div>
    </div>
EOT;
  }
}


function _dqx_adminmenu_build_menu_item($item) {
  if ($item['type'] == MENU_CALLBACK) {
    $item['hidden'] = -1;
  }
  elseif ($item['type'] == MENU_SUGGESTED_ITEM) {
    $item['hidden'] = 1;
  }
  $item['module'] = 'adminmneu';
  $item += array(
    'menu_name' => 'dqx_adminmenu',
    'link_title' => $item['title'],
    'link_path' => $item['path'],
    'hidden' => 0,
    'options' => empty($item['description']) ? array() : array(
      'attributes' => array('title' => $item['description'])
    ),
    
    'router_path' => $item['path'],
    
    // this is taken from menu_item_save().
    'weight' => 0,
    'has_children' => 0,
    'expanded' => 1,
    'customized' => 0,
    'updated' => 0,
  );
  return $item;
}


function _dqx_adminmenu_items_check_access(&$items) {
  foreach ($items as $path => &$item) {
    _menu_link_translate($item);
  }
  // Sort siblings in the tree based on the weights and localized titles.
  // ksort($new_tree);
}


function _dqx_adminmenu_path_multisplit(array &$multisplit, $path, array $prefix_map) {
  $fragments = explode('/', $path);
  $n = count($fragments);

  $i=1;
  $suffix = array_pop($fragments);
  while ($fragments) {
    $prefix = implode('/', $fragments);
    if (isset($prefix_map[$prefix])) {
      $prefix = $prefix_map[$prefix];
      $multisplit[$prefix][$path] = $path;
      return;
    }
    $suffix = array_pop($fragments) .'/'. $suffix;
    ++$i;
  }

  $multisplit[NULL][$path] = $path;
}



class _dqx_adminmenu_MenuTree {
  protected $_submenus;
  protected $_items;

  function __construct(array $submenus, array $items) {
    $this->_submenus = $submenus;
    $this->_items = $items;
  }

  function renderMenu($parent_path, $id = NULL) {
    return $this->_renderSubmenu($parent_path, $id, 0);
  }

  function pathAccess($path) {
    $item = $this->_items[$path];
    if ($item && isset($item['access'])) {
      return $item['access'];
    }
    else {
      $router_item = menu_get_item($path);
      return $router_item['access'];
    }
  }

  protected function _renderSubmenu($parent_path, $id, $depth) {
    $html = '';
    $submenu = $this->_submenus[$parent_path];
    if (is_object($submenu)) {
      return $submenu->render($this);
    }
    else if (is_array($submenu)) {
      $sort = array();
      $pieces = array();
      foreach ($submenu as $k => $path) {
        if ($item = $this->_items[$path]) {
          $options = $item['localized_options'];
          $options = is_array($options) ? $options : array();
          $subtree_html = $this->_renderSubmenu($path, NULL, $depth + 1);
          if (!isset($item['access'])) {
            // late access check
            $router_item = menu_get_item($item['link_path']);
            $item['access'] = $router_item['access'];
          }
          if ($item['access']) {
            $link_html = l($item['title'], $item['link_path'], $options);
            $class = $subtree_html ? ' class="expandable"' : '';
            $pieces[$k] = '<li'. $class .'>'. $link_html . $subtree_html .'</li>';
            $sort[$k] = (50000 + $item['weight']) .' '. $item['title'];
          }
          else if ($subtree_html) {
            // no access, but still want to reveal some of the subtree items
            $link_html = $options['html'] ? $item['title'] : check_plain($item['title']);
            $link_html = '<a class="no-access">'. $link_html .'</a>';
            $class = $subtree_html ? ' class="expandable no-access"' : ' class="no-access"';
            $pieces[$k] = '<li'. $class .'>'. $link_html . $subtree_html .'</li>';
            $sort[$k] = (50000 + $item['weight']) .' '. $item['title'];
          }
        }
      }
      if ($depth > 0 || TRUE) {
        array_multisort($sort, $pieces);
      }
    }
    $id = $id ? (' id="'. $id .'"') : '';
    return count($pieces) ? ('<ul'. $id .'>'. implode('', $pieces) .'</ul>') : '';
  }
}



/**
 * This only makes sense in combination with admin_views.
 * I recommend this module anyway, so we assume that it is enabled.
 */
class _dqx_adminmenu_ContentTablePlugin {

  protected function _getColumns() {
    $cols = array();
    $cols['list'] = 'List';
    $cols['create'] = 'Create';
    if (user_access('administer content types')) {
      $cols['space'] = '';
      $cols['configure'] = 'Configure';
      if (module_exists('content')) {
        $cols['fields'] = 'Fields';
        if (!module_exists('nd')) {
          // With nd enabled, the normal 'display' would be confusing.
          // It is still enabled in the submenu.
          $cols['display'] = 'Display';
        }
      }
      $cols['submenu'] = '';
    }
    return $cols;
  }

  protected function _buildMatrix(array $types, array &$cols, _dqx_adminmenu_MenuTree $tree) {
    $cols_cellcount = array();
    $matrix = array();
    foreach ($types as $type) {
      $type->slug = str_replace('_', '-', $type->type);
      $type->class = 'expandable node-type node-type-'. $type->type;
      foreach ($cols as $key => $title) {
        $method = '_cell__'. $key;
        if ($key === 'space') {
          $matrix[$type->type]['space'] = '';
        }
        else if (method_exists($this, $method)) {
          $cell = $this->$method($type, $tree);
          if ($cell) {
            ++ $cols_cellcount[$key];
            $matrix[$type->type][$key] = $cell;
          }
        }
      }
    }
    foreach ($cols_cellcount as $key => $count) {
      if (!$count) {
        unset($cols[$key]);
      }
    }
    $keys_reverse = array_reverse(array_keys($cols));
    if ($keys_reverse[0] === 'space') {
      unset($cols['space']);
    }
    if ($keys_reverse[0] === 'submenu' && $keys_reverse[1] === 'space') {
      unset($cols['space']);
    }
    return $matrix;
  }

  protected function _renderFooter(array $cols, _dqx_adminmenu_MenuTree $tree) {
    $html = '';
    $colspan = 0;
    $add_type = FALSE;
    foreach ($cols as $key => $title) {
      ++ $colspan;
      if ($key === 'space') {
        $html .= '<td class="td-space-colspan" colspan="'. $colspan .'">&nbsp;</td>';
        $add_type = TRUE;
        $colspan = 0;
      }
    }
    if ($colspan) {
      if ($add_type) {
        $link = l('Add type', 'admin/content/type/add');
        $html .= '<td class="admin-content-type-add" colspan="'. $colspan .'">'. $link .'</td>';
      }
      else {
        $html .= '<td colspan="'. $colspan .'"></td>';
      }
    }
    if ($html && $tree->pathAccess('admin/content/type/add')) {
      return '<tfoot><tr>'. $html .'</tr></tfoot>';
    }
  }

  function render(_dqx_adminmenu_MenuTree $tree) {
    $cols = $this->_getColumns();
    $types = array();
    foreach (node_get_types() as $type) {
      $types[$type->type] = clone $type;
    }
    
    $matrix = $this->_buildMatrix($types, $cols, $tree);
    
    $html = '';
    foreach ($types as $type) {
      $row_html = '';
      foreach ($cols as $key => $title) {
        $class = 'td-'. $key;
        $cell = $matrix[$type->type][$key];
        $row_html .= '<td class="'. $class .'">'. $cell .'</td>';
      }
      $html .= '<tr class="'. $type->class .'">'. $row_html .'</tr>';
    }
    $tbody = '<tbody>'. $html .'</tbody>';
    
    $tfoot = $this->_renderFooter($cols, $tree);
    return '<table>'. $tfoot . $tbody .'</table>';
  }

  protected function _cell__list($type) {
    return l('list: <strong>'. $type->name .'</strong>', 'admin/content/node', array(
      'query' => array('type' => $type->type),
      'html' => TRUE,
    ));
  }

  protected function _cell__create($type) {
    return l('create', 'node/add/'. $type->slug);
  }

  protected function _cell__configure($type) {
    return l('configure', 'admin/content/node-type/'. $type->slug);
  }

  protected function _cell__fields($type) {
    return l('fields', 'admin/content/node-type/'. $type->slug .'/fields');
  }

  protected function _cell__display($type) {
    return l('display', 'admin/content/node-type/'. $type->slug .'/display');
  }
  
  protected function _cell__submenu($type, $tree) {
    $submenu_key = 'admin/content/node-type/'. $type->slug;
    $submenu = $tree->renderMenu($submenu_key);
    return '<div>'. $submenu .'</div>';
  }
}



/**
 * Build user/action links; mostly account information and links.
 *
 * @see theme_admin_menu_links()
 */
function _dqx_adminmenu_user_items() {
  if (!$GLOBALS['user']->uid) {
    // Not logged in.
    // Anonymous is not supposed to get access to the dqx_adminmenu,
    // but let's do something safe nevertheless.
    return array();
  }
  // dummy parent item
  $items['dqx_adminmenu-user'] = array();
  $items['dqx_adminmenu-user/counter'] = array(
    'title' => _dqx_adminmenu_get_user_count(),
    'weight' => -90,
    'localized_options' => array('attributes' => array(
      'id' => 'dqx_adminmenu-user-counter',
      'title' => t('Current anonymous / authenticated users'),
    )),
    'link_path' => (user_access('administer users') ? 'admin/user/user' : 'user'),
    'access' => TRUE,
  );
  $items['dqx_adminmenu-user/account'] = array(
    'title' => $GLOBALS['user']->name,
    'weight' => -99,
    'localized_options' => array(
      'attributes' => array('id' => 'dqx_adminmenu-user-account'),
    ),
    'link_path' => 'user/' . $GLOBALS['user']->uid,
    'access' => TRUE,
  );
  $items['dqx_adminmenu-user/logout'] = array(
    'title' => t('Log out'),
    'weight' => -100,
    'attributes' => array('id' => 'dqx_adminmenu-user-logout'),
    'link_path' => 'logout',
    'access' => TRUE,
  );

  return $items;
}


function _dqx_adminmenu_get_user_count() {
  $interval = time() - variable_get('user_block_seconds_online', 900);
  $count_anon = sess_count($interval);
  $count_auth = db_result(db_query("SELECT COUNT(DISTINCT uid) FROM {sessions} WHERE uid > 0 AND timestamp >= %d", $interval));

  return t('@count-anon / @count-auth', array('@count-anon' => $count_anon, '@count-auth' => $count_auth));
}



/**
 * Build icon menu links; mostly containing maintenance helpers.
 *
 * @see theme_admin_menu_links()
 */
function _dqx_adminmenu_icon_items() {
  
  $icon_src = theme_get_setting('toggle_favicon') ? theme_get_setting('favicon') : base_path() . 'misc/favicon.ico';
  $icon_img = '<img id="dqx_adminmenu-icon-img" src="'. $icon_src .'" width="16" height="16" alt="' . t('Home') . '" />';

  $items = array();
  $items['admin/dqx_adminmenu'] = array(
    'title' => $icon_img,
    'access' => TRUE,  // everyone should see the front page
    'weight' => -100,
    'link_path' => '<front>',
    'localized_options' => array(
      'attributes' => array('id' => 'dqx_adminmenu-icon-link'),
      'html' => TRUE,
    ),
  );
  // Add link to manually run cron.
  $items['admin/dqx_adminmenu/cron'] = array(
    'title' => t('Run cron'),
    'weight' => 50,
    'access' => user_access('administer site configuration'),
    'link_path' => 'admin/reports/status/run-cron',
  );
  // Add link to run update.php.
  $items['admin/dqx_adminmenu/update'] = array(
    'title' => t('Run updates'),
    'weight' => 50,
    'access' => ($GLOBALS['user']->uid == 1 || !empty($GLOBALS['update_free_access'])),
    'link_path' => base_path() . 'update.php',
    'localized_options' => array(
      'external' => TRUE,
    ),
  );
  // Add items to flush caches.
  $items['admin/dqx_adminmenu/flush-cache'] = array(
    'title' => t('Flush all caches'),
    'weight' => 20,
    // @todo Add permission to flush cashes.
    'access' => user_access('administer site configuration'),
    'link_path' => 'admin/dqx_adminmenu/flush-cache',
    'localized_options' => array(
      'attributes' => array('class' => 'dqx_adminmenu-redirect'),
    ),
  );
  $caches = array(
    // 'admin_menu' => t('Administration menu'),
    'cache' => t('Cache tables'),
    'menu' => t('Menu'),
    'requisites' => t('Page requisites'),
    'theme' => t('Theme registry'),
  );
  foreach ($caches as $arg => $title) {
    $items['admin/dqx_adminmenu/flush-cache/'. $arg] = array(
      'title' => $title,
      'link_path' => 'admin/dqx_adminmenu/flush-cache/' . $arg,
      'localized_options' => array(
        'attributes' => array('class' => 'dqx_adminmenu-redirect'),
      ),
      'access' => user_access('administer site configuration'),
    );
  }
  // Add link to toggle developer modules (performance).
  $saved_state = variable_get('admin_menu_devel_modules_enabled', NULL);
  $items['admin/dqx_adminmenu/toggle-modules'] = array(
    'title' => isset($saved_state) ? t('Enable developer modules') : t('Disable developer modules'),
    'weight' => 88,
    'access' => user_access('administer site configuration'),
    'link_path' => 'admin/dqx_adminmenu/toggle-modules',
    'localized_options' => array(
      'attributes' => array('class' => 'dqx_adminmenu-redirect'),
    ),
  );

  // Add Devel module links.
  if (module_exists('devel')) {
    // Add variable editor.
    $items['admin/dqx_adminmenu/devel-variables'] = array(
      'title' => t('Variable editor'),
      'weight' => 20,
      'access' => user_access('access devel information'),
      'link_path' => 'devel/variable',
    );
  }

  return $items;
}


function _dqx_adminmenu_menu_editor_items() {
  $items = array();
  if (module_exists('menu_editor')) {
    $q = db_query("SELECT * FROM {menu_custom} ORDER BY title");
    while ($menu = db_fetch_array($q)) {
      $path = 'admin/build/menu-customize/'. $menu['menu_name'] .'/poweredit';
      $items[$path] = array(
        'title' => $menu['title'],
        'link_path' => $path,
        'weight' => 100,
      );
    }
  }
  return $items;
}




